import os
import numpy as np
from math import floor
import math
import mrcfile
from va.utils.misc import *
from va.utils.Model import *
from va.metrics.contour_level_predicator import *


class MapProcessor:
    """
        MapProcessor class contains methods deal with map processing method and model associated map processing methods
        Instance can be initialized with either full map file path or a mrcfile map object
    """

    def __init__(self, input_map=None):
        if isinstance(input_map, str):
            if os.path.isfile(input_map):
                self.map = mrcfile.open(input_map)
            else:
                self.map = None
        elif isinstance(input_map, mrcfile.mrcfile.MrcFile):
            self.map = input_map
        else:
            self.map = None

    def get_indices(self, one_coord):
        """
            Find one atom's indices corresponding to its cubic or plane
            the 8 (cubic) or 4 (plane) indices are saved in indices variable

        :param one_coord: List contains the atom coordinates in (x, y, z) order
        :return: Tuple contains two list of index: first has the 8 or 4 indices in the cubic;
                 second has the float index of the input atom
        """
        # For non-cubic or skewed density maps, they might have different apix on different axes
        zdim = self.map.header.cella.z
        znintervals = self.map.header.mz
        z_apix = zdim / znintervals

        ydim = self.map.header.cella.y
        ynintervals = self.map.header.my
        y_apix = ydim / ynintervals

        xdim = self.map.header.cella.x
        xnintervals = self.map.header.mx
        x_apix = xdim / xnintervals

        map_zsize = self.map.header.nz
        map_ysize = self.map.header.ny
        map_xsize = self.map.header.nx

        if self.map.header.cellb.alpha == self.map.header.cellb.beta == self.map.header.cellb.gamma == 90.:
            zindex = float(one_coord[2] - self.map.header.origin.z) / z_apix - self.map.header.nzstart
            yindex = float(one_coord[1] - self.map.header.origin.y) / y_apix - self.map.header.nystart
            xindex = float(one_coord[0] - self.map.header.origin.x) / x_apix - self.map.header.nxstart

        else:
            # fractional coordinate matrix
            xindex, yindex, zindex = self.matrix_indices(one_coord)

        zfloor = int(floor(zindex))
        if zfloor >= map_zsize - 1:
            zceil = zfloor
        else:
            zceil = zfloor + 1

        yfloor = int(floor(yindex))
        if yfloor >= map_ysize - 1:
            yceil = yfloor
        else:
            yceil = yfloor + 1

        xfloor = int(floor(xindex))
        if xfloor >= map_xsize - 1:
            xceil = xfloor
        else:
            xceil = xfloor + 1

        indices = np.array(np.meshgrid(np.arange(xfloor, xceil + 1), np.arange(yfloor, yceil + 1),
                                       np.arange(zfloor, zceil + 1))).T.reshape(-1, 3)
        oneindex = [xindex, yindex, zindex]

        return (indices, oneindex)

    def matrix_indices(self, onecoor):
        """
            using the fractional coordinate matrix to calculate the indices when the maps are non-orthogonal

        :param onecoor: list contains the atom coordinates in (x, y, z) order
        :return: tuple of indices in x, y, z order
        """

        # Figure out the order of the x, y, z based on crs info in the header
        apixs = self.map.voxel_size.tolist()
        angs = [self.map.header.cellb.alpha, self.map.header.cellb.beta, self.map.header.cellb.gamma]
        matrix = self.map_matrix(apixs, angs)
        result = matrix.dot(np.asarray(onecoor))
        xindex = result[0] - self.map.header.nxstart
        yindex = result[1] - self.map.header.nystart
        zindex = result[2] - self.map.header.nzstart

        return xindex, yindex, zindex

    @staticmethod
    def map_matrix(apixs, angs):
        """
            calculate the matrix to transform Cartesian coordinates to fractional coordinates
            (check the definition to see the matrix formular)

        :param apixs: array of apix/voxel size
        :param angs: array of angles in alpha, beta, gamma order
        :return: a numpy array to be used for calculated fractional coordinates
        """

        ang = (angs[0] * math.pi / 180, angs[1] * math.pi / 180, angs[2] * math.pi / 180)
        insidesqrt = 1 + 2 * math.cos(ang[0]) * math.cos(ang[1]) * math.cos(ang[2]) - \
                     math.cos(ang[0]) ** 2 - \
                     math.cos(ang[1]) ** 2 - \
                     math.cos(ang[2]) ** 2

        cellvolume = apixs[0] * apixs[1] * apixs[2] * math.sqrt(insidesqrt)

        m11 = 1 / apixs[0]
        m12 = -math.cos(ang[2]) / (apixs[0] * math.sin(ang[2]))

        m13 = apixs[1] * apixs[2] * (math.cos(ang[0]) * math.cos(ang[2]) - math.cos(ang[1])) / (
                    cellvolume * math.sin(ang[2]))
        m21 = 0
        m22 = 1 / (apixs[1] * math.sin(ang[2]))
        m23 = apixs[0] * apixs[2] * (math.cos(ang[1]) * math.cos(ang[2]) - math.cos(ang[0])) / (
                    cellvolume * math.sin(ang[2]))
        m31 = 0
        m32 = 0
        m33 = apixs[0] * apixs[1] * math.sin(ang[2]) / cellvolume
        prematrix = [[m11, m12, m13], [m21, m22, m23], [m31, m32, m33]]
        matrix = np.asarray(prematrix)

        return matrix

    def get_close_voxels_indices(self, onecoor, n):
        """
            Given onecoor, return the nearby voxels indices; radius defined by n

        :param onecoor: list contains the atom coordinates in (x, y, z) order
        :param n: a number integer of float which define radius voxel check range radius = (n*average_voxel_size)
        :return: a list of tuples of indices in (z, y, x) format to adapt to mrcfile data format
        """

        xind, yind, zind = self.get_indices(onecoor)[1]
        voxel_sizes = self.map.voxel_size.tolist()
        atom_xind = int(xind)
        atom_yind = int(yind)
        atom_zind = int(zind)

        average_voxel_size = sum(voxel_sizes) / 3.
        radius = n * average_voxel_size
        rx = int(round(radius / voxel_sizes[0]))
        ry = int(round(radius / voxel_sizes[1]))
        rz = int(round(radius / voxel_sizes[2]))

        indices = []
        for x in range(atom_xind - rx, atom_xind + rx):
            for y in range(atom_yind - ry, atom_yind + ry):
                for z in range(atom_zind - rz, atom_zind + rz):
                    d = average_voxel_size * math.sqrt(
                        (x - atom_xind) ** 2 + (y - atom_yind) ** 2 + (z - atom_zind) ** 2)
                    if d <= radius:
                        indices.append([x, y, z])
        result = [tuple(x[::-1]) for x in indices]

        return result

    def generate_mask(self, coords, radius):
        """
            Based on the coordinates, generate a mask based on the radius
            The mask based on the map initialized in the MapProcessor class

        :param coords: a list of tuples in (x, y, z) format
        :param radius: an integer or float define the radius of mask range around the coordinate
        """

        dir_name = os.path.dirname(self.map._iostream.name)
        map_name = os.path.basename(self.map._iostream.name)
        mask = np.zeros_like(self.map.data)
        for coord in coords:
            near_indices = self.get_close_voxels_indices(coord, radius)
            for ind in near_indices:
                mask[ind] = 1
        out_map = mrcfile.new(f'{dir_name}/{map_name}_residue_mask.mrc', overwrite=True)
        out_map.set_data(mask)
        out_map.voxel_size = self.map.voxel_size
        out_map.close()

    def residue_average_resolution(self, indices, mapdata=None):
        """
            given mapdata and indices, calculate the average value of these density values

        :param mapdata: numpy array of map data
        :param indices: list of tuples of (x, y, z) coordinates
        return: average value of these density values
        """

        sum_local_resolution = 0.
        if mapdata is None:
            mapdata = self.map.data
        for ind in indices:
            sum_local_resolution += mapdata[ind]

        return sum_local_resolution / len(indices)

    @staticmethod
    def save_map(map_data, output_mapname, voxel_size=1.):
        """
            save to a new map
        :param map_data: np array of map data
        :param output_mapname: full path to output map
        :param voxel: voxel size use default as 1 if not given
        """

        m = mrcfile.new(output_mapname, overwrite=True)
        m.set_data(map_data)
        m.voxel_size = voxel_size
        m.close()

    @staticmethod
    def get_map_volume(full_input_map_name, contour=1.0):
        """
            Collect the map volume
        :param full_input_map_name: string of full path to input map
        :param contour: contour value if not provided use 1.0 as default
        """

        input_map = mrcfile.mmap(full_input_map_name, mode='r')
        # apix = input_map.voxel_size.tolist()
        voxel_volume = np.prod(input_map.voxel_size.tolist())
        voxel_count = np.sum(input_map.data >= contour)
        map_volume = voxel_volume * voxel_count

        return map_volume


    @staticmethod
    def mask_map(input_map, mask, output_mapname=None):
        """
            Mask the input map
        :param input_map: string of full path to input map
        :param mask: string of full path to mask
        :param output_mapname: full path to output map name
        """

        in_map = mrcfile.open(input_map)
        in_map_name = os.path.basename(input_map)
        in_map_data = in_map.data
        mask_map = mrcfile.open(mask)
        mask_data = mask_map.data
        mask_name = os.path.basename(mask)
        work_dir = os.path.dirname(input_map)
        if output_mapname is None:
            output_mapname = f'{work_dir}/{in_map_name}_{mask_name}_masked.map'
        if in_map_data.shape != mask_data.shape and input_map.voxel_size == mask.voxel_size:
            print(f'Map shape mismatch: {in_map_data.shape} and {mask_data.shape} or '
                  f'voxel size mismatch: {input_map.voxel_size} and {mask_data.voxel_size}')

            return None
        else:
            out_data = in_map_data * mask_data
            voxel = in_map.voxel_size
            MapProcessor.save_map(out_data, output_mapname, voxel)

            return output_mapname

    @staticmethod
    def binarized_mask(mask, map_name):
        """
            Produce a mask with 0 and 1s (for Relion mask with value > 0.5)
        :param mask: a string of full path to a mask
        :param map_name: a string of full path of primary map related to this mask
        """

        mask_map = mrcfile.open(mask)
        mask_map_data = mask_map.data > 0.5
        new_data = mask_map_data.astype(np.uint8)
        voxel_size = mask_map.voxel_size
        work_dir = os.path.dirname(os.path.dirname(os.path.dirname(mask)))
        outmap_name = f'{work_dir}/{map_name}_binarized_mask.map'
        MapProcessor.save_map(new_data, outmap_name, voxel_size)

        return outmap_name

    @staticmethod
    def predict_contour(input_map):
        """
            Given input map, predict the contour leve
        :param input_map: a string of full input map path
        :return: a float of the contour
        """

        m = mrcfile.open(input_map)
        d = m.data
        # non-cubic map use padding 0 to make it cubic
        if not all(dim == d.shape[0] for dim in d.shape):
            dim_shape = max(d.shape)
            target_shape = (dim_shape, dim_shape, dim_shape)
            d = pad_array(m.data, target_shape)
        norm_pred = calc_level_dev(d)[0]
        pred_cl = keep_three_significant_digits(float(norm_pred))

        return pred_cl


    @staticmethod
    def map_indices(input_map, contour):
        """
            Given input map and contour return all indices that correspond to the contour value larger than contour
        :param input_map: a string of full input map path
        :param contour: a float value of the contour
        :return: a list of tuples in (x, y, z)
        """
        map = mrcfile.open(input_map)

        return np.where(map.data >= contour)

    @staticmethod
    def map_minmax(map_one, map_two):
        """
            Get input map min and max value
        :param map_one(masked_raw_map): a full path of input map one
        :param map_two: a full path of input map two
        """

        masked_raw_predicted_contour = MapProcessor.predict_contour(map_one)
        all_indices = MapProcessor.map_indices(map_one, masked_raw_predicted_contour)
        local_res_map = mrcfile.open(map_two)
        all_values = local_res_map.data[all_indices]

        return np.min(all_values), np.max(all_values)


    def model_area_indices(self, input_map, model, radius=3):
        """
            Get all indices of the 'mask' that used for the area around the model in the map
        """
        map = MapProcessor(input_map)
        # use for generate mask for the whole model or the voxels involved
        all_indices = set()
        # all_coordinates = []
        for chain in model.get_chains():
            for residue in chain.get_residues():
                residue_atom_count = 0
                for atom in residue.get_atoms():
                    if atom.name.startswith('H') or atom.get_parent().resname == 'HOH':
                        continue
                    one_coordinate = atom.coord
                    around_indices = map.get_close_voxels_indices(one_coordinate, radius)
                    # use for generate mask for the whole model or the voxels involved
                    # all_coordinates.append(one_coordinate)
                    all_indices.update(around_indices)
                if residue_atom_count == 0:
                    continue

        # Get model volume based on voxel size and indices numbers
        voxel_volume = np.prod(map.map.voxel_size.tolist())
        model_volume = len(all_indices) * voxel_volume

        return all_indices, len(all_indices), model_volume



    def model_ratio(self, input_map, model, radius=3):
        """
            Check the percentage of a model that covered the input map
        :param input_map: a string of full input map path
        :param model: a string of full model path
        :param radius: distance to the atom which defined the model area, default as 3
        """

        work_dir = None

        if isinstance(input_map, str):
            work_dir = os.path.dirname(input_map)
        elif isinstance(input_map, mrcfile.mrcfile.MrcFile):
            input_map = input_map._iostream.name
            work_dir = os.path.dirname(input_map)
        model_container = Model(model, work_dir)
        # model_container = Model('/Users/zhe/Downloads/tests/VA/nfs/msd/work2/emdb/development/staging/em/81/8117/va/5irx.cif')
        loaded_model = model_container.final_model()
        model_area_indices, model_area_indices_number, model_volume = self.model_area_indices(input_map, loaded_model, radius)
        # Save model volume into json
        out_file = f'{work_dir}/{os.path.basename(input_map)}_modelvolume.json'
        model_volume_dict = {'model_volume': {'volume': model_volume, 'radius': radius}}
        out_json(model_volume_dict, out_file)

        ## code for model area into a map
        # mask = np.zeros_like(self.map.data)
        # for i in model_area_indices:
        #     mask[i] = 1
        # out_map = mrcfile.new(f'{work_dir}/model_area.mrc', overwrite=True)
        # out_map.set_data(mask)
        # out_map.voxel_size = self.map.voxel_size
        # out_map.close()
        ##
        predicated_contour_level = self.predict_contour(input_map)
        map_area = self.map_indices(input_map, predicated_contour_level)
        map_area_indices = list(zip(map_area[0], map_area[1], map_area[2]))
        overlapped_area = set(map_area_indices) & set(model_area_indices)
        overlapped_area_size = len(list(overlapped_area))
        # ### code for put overlap region as a map
        # mask = np.zeros_like(self.map.data)
        # for i in overlapped_area:
        #     mask[i] = 1
        # out_map = mrcfile.new(f'{work_dir}/overlap.mrc', overwrite=True)
        # out_map.set_data(mask)
        # out_map.voxel_size = self.map.voxel_size
        # out_map.close()
        # ##
        overlap_to_model = keep_three_significant_digits(overlapped_area_size/model_area_indices_number)
        overlap_to_map = keep_three_significant_digits(overlapped_area_size/len(map_area_indices))
        model_to_map = keep_three_significant_digits(model_area_indices_number/len(map_area_indices))
        print(f'overlap/model: {overlap_to_model}')
        print(f'overlap/map: {overlap_to_map}')
        print(f'model/map: {model_to_map}')
        final_result = {'model_map_ratio': {'overlap_to_model': overlap_to_model, 'overlap_to_map': overlap_to_map,
                                            'model_to_map': model_to_map}}
        out_file = f'{work_dir}/{os.path.basename(input_map)}_modelmapratio.json'
        out_json(final_result, out_file)

        return out_file









import os.path

import numpy as np
import matplotlib.pyplot as plt
# from statsmodels.nonparametric.smoothers_lowess import lowess
from scipy.signal import find_peaks
from va.utils.misc import out_json
from PIL import Image

class FSCChecks:
    def __init__(self, input_data):
        try:
            self.fsc_data = input_data['curves']['fsc'][5:]
        except KeyError as e:
            raise Exception(f"Error: Required data not found in the JSON file. {e}")

    def min_value(self):
        return np.min(self.fsc_data)

    def end_value(self):
        return self.fsc_data[-1]

    def min_final_diff_value(self):
        return self.end_value() - self.min_value()

    def large_drop_check(self, window_size=5, drop_threshold=0.7):
        max_gradient_change = 0
        for i in range(window_size, len(self.fsc_data)):
            window_data = self.fsc_data[i - window_size: i]
            if abs(window_data[-1] - window_data[0]) > drop_threshold:
                differences = np.diff(window_data)
                average_difference = np.mean(differences)
                max_gradient_change = max(max_gradient_change, abs(average_difference))
        return max_gradient_change

    def max_gradient_check(self, window_size=5):
        max_change = float('-inf')
        for i in range(window_size, len(self.fsc_data)):
            window_data = self.fsc_data[i - window_size: i]
            differences = np.diff(window_data)
            average_difference = np.mean(differences)
            max_change = max(max_change, abs(average_difference))
        return max_change

    def peak_finder(self, window_size=5):
        # smoothed_data = lowess(self.fsc_data, range(len(self.fsc_data)), frac=0.05)[:, 1]
        smoothed_data = np.convolve(self.fsc_data, np.ones(window_size) / window_size, mode='valid')
        peaks, _ = find_peaks(smoothed_data, distance=5, prominence=0.1)
        return len(peaks)

    def fsc_plotter(self, filepath):
        plt.figure()
        plt.plot(range(len(self.fsc_data)), self.fsc_data)
        plt.axhline(y=0, color='red', linestyle='--', label='Zero Line')
        plt.axhline(y=0.143, color='orange', linestyle='--', label='0.143 Line')
        plt.title(filepath)
        plt.savefig('{}.png'.format(filepath))
        plt.close()

    def fsc_checks(self, output_json):
        fsc_check = {}
        fsc_check['FSC'] = {}
        fsc_check['FSC']['minValue'] = self.min_value()
        fsc_check['FSC']['endValue'] = self.end_value()
        fsc_check['FSC']['peaks'] = self.peak_finder()
        fsc_check['FSC']['largestGradient'] = self.max_gradient_check()
        out_json(fsc_check, output_json)


class ImageChecks:
    """
        This class is designed to run checks on the VA images.
        Each check should be added as a function and then called in RunChecksPerCheck.py
    """

    def __init__(self, input_image):
        try:
            self.input_image = input_image
            self.image_dir = os.path.dirname(input_image)
        except KeyError as e:
            raise Exception(f"Error: Required data not found in the JSON file. {e}")

    def mask_check(self):

        img = Image.open(self.input_image)
        img_array = np.array(img)
        green_pixels = np.sum(np.all(img_array == [0, 138, 0], axis=-1))
        total_pixels = img_array.shape[0] * img_array.shape[1]
        proportion_green = (green_pixels / total_pixels) * 100

        # Calculate proportions of green for halves split vertically
        mid_vertical = img_array.shape[1] // 2
        green_pixels_left_half = np.sum(np.all(img_array[:, :mid_vertical, :] == [0, 138, 0], axis=-1))
        green_pixels_right_half = np.sum(np.all(img_array[:, mid_vertical:, :] == [0, 138, 0], axis=-1))
        proportion_green_left_half = (green_pixels_left_half / (total_pixels // 2)) * 100
        proportion_green_right_half = (green_pixels_right_half / (total_pixels // 2)) * 100
        diff_vertical = proportion_green_left_half - proportion_green_right_half

        # Calculate proportions of green for halves split horizontally
        mid_horizontal = img_array.shape[0] // 2
        green_pixels_top_half = np.sum(np.all(img_array[:mid_horizontal, :, :] == [0, 138, 0], axis=-1))
        green_pixels_bottom_half = np.sum(np.all(img_array[mid_horizontal:, :, :] == [0, 138, 0], axis=-1))
        proportion_green_top_half = (green_pixels_top_half / (total_pixels // 2)) * 100
        proportion_green_bottom_half = (green_pixels_bottom_half / (total_pixels // 2)) * 100
        diff_horizontal = proportion_green_top_half - proportion_green_bottom_half

        return proportion_green, diff_vertical, diff_horizontal

    def image_check(self):

        try:
            proportion_green, diff_vertical, diff_horizontal = self.mask_check()
            result_dict = {'proportion_green': proportion_green, 'maskDiffVertical': diff_vertical,
                           'maskDiffHorizontal': diff_horizontal}
            final_dict = {'imageChecks': result_dict}
            output_file = f'{self.image_dir}/checks/image.json'
            out_json(final_dict, output_file)
        except Exception as e:
            print(f'The image check error is: {e}')





# Fast coloring of residues that avoids slow atom spec parsing.  Atom spec must have form /{chain_id}:{resnumber}.
#
#     rescolor /A:123 blue
#
# or interleaved residues and colors on a line (colors must be in format #rrggbb)
#
#     rescolors /A0:14 #3737ff /A0:15 #3535ff /A0:16 #3232ff


def rescolor(session, res_spec, color):
    parts = res_spec.split(':')
    chain_id = parts[0][1:]
    res_number = int(parts[1])
    from chimerax.atomic import AtomicStructure
    for s in session.models.list(type = AtomicStructure):
        if not hasattr(s, 'res_table'):
            s.res_table = {(r.chain_id, r.number):r for r in s.residues}
        r = s.res_table[(chain_id, res_number)]
        r.color = color
        r.atoms.colors = color
        # color ribbon
        r.ribbon_color = color


def rescolors(session, res_color_string):
    res_color_list = res_color_string.split()
    for res_spec, color in zip(res_color_list[::2], res_color_list[1::2]):
        rgba = tuple(int(color[i:i+2], 16) for i in range(1, 7, 2)) + (255,)
        rescolor(session, res_spec, rgba)


def register_command(session):
    from chimerax.core.commands import CmdDesc, register, StringArg, Color8Arg, RestOfLine
    desc = CmdDesc(required=[('res_spec', StringArg),
                              ('color', Color8Arg)],
                   synopsis='Color a residue fast')
    register('rescolor', desc, rescolor, logger=session.logger)
    desc = CmdDesc(required=[('res_color_string', RestOfLine),],
                   synopsis='Color residues fast')
    register('rescolors', desc, rescolors, logger=session.logger)

register_command(session)


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches


def load_score(input_file, new_entry, score_type):
    """
    Load score into dataframe
    """

    type_dict = {'id': str, 'resolution': float, 'name': str, score_type: float}
    df_nonew = pd.read_csv(input_file, dtype=type_dict)
    new_entry_df = pd.DataFrame([new_entry])
    df = pd.concat([df_nonew, new_entry_df], ignore_index=True)
    df_without_nan = df[~df.isnull().any(axis=1)].sort_values(by=score_type).reset_index()
    subsets = ['id', 'resolution', 'name', score_type]
    df_without_nan = df_without_nan.drop_duplicates(subsets)

    return df_without_nan


def get_score(df, input_value, score_type):
    """
    Given the dataframe, return a tuple one contain min and max of the score and
    the other contains the number of items smaller than the input value and number
    of items larger than the input value
    """

    qmin = None
    qmax = None
    small = None
    large = None
    if not df.empty:
        qmin = df[score_type].min()
        qmax = df[score_type].max()
        large = df[df[score_type] > input_value].shape[0]
        small = df[df[score_type] <= input_value].shape[0]

    return (qmin, qmax), (small, large)


def match_to_newscale(original_scale, target_scale, original_value):
    """
    Scale the original_value based in the original_scale to target_scale
    """

    original_min = original_scale[0]
    original_max = original_scale[1]

    target_min = target_scale[0]
    target_max = target_scale[1]

    target_value = ((original_value - original_min) / (original_max - original_min)) * (
                target_max - target_min) + target_min

    return target_value


def get_nearest_onethousand(new_entry, df, n, score_type):
    """
    Sort the score based on resolution and then return the nearest 1000 center at the
    new_entry
    """

    # Find the index of the nearest row to the target value
    cdf = df.sort_values(by='resolution').reset_index()
    # nearest_index = (cdf['qscore'] - new_entry['qscore']).abs().idxmin()
    # nearest_index = cdf[cdf['id'] == new_entry['id']].index.to_list()[0]
    # As the new entry added into the df, use all the 4 columns to identify the row index
    mask = (cdf['id'] == new_entry['id']) & (cdf['resolution'] == new_entry['resolution']) & (
                cdf['name'] == new_entry['name']) & (cdf[score_type] == new_entry[score_type])
    nearest_index = cdf[mask].index[0]

    # Get the 1000 rows centered around the nearest index
    start = max(nearest_index - n, 0)
    end = min(nearest_index + n, len(df))
    df_nearest = cdf.iloc[start:end + 1]

    return df_nearest


def get_resolution_range(new_entry, df, score_type, resbin=1.):
    """
    Sort the df based on the resolution and then find the nearest +1-1 resolution df
    """

    if new_entry['resolution']:
        # Find the index of the nearest row to the target value
        cdf = df.sort_values(by='resolution').reset_index()
        # nearest_index = (cdf['qscore'] - new_entry['qscore']).abs().idxmin()
        # nearest_index = cdf[cdf['id'] == new_entry['id']].index.to_list()[0]
        # As the new entry added into the df, use all the 4 columns to identify the row index
        mask = (cdf['id'] == new_entry['id']) & (cdf['resolution'] == new_entry['resolution']) & (
                    cdf['name'] == new_entry['name']) & (cdf[score_type] == new_entry[score_type])
        nearest_index = cdf[mask].index

        # Get the 1000 rows centered around the nearest index
        start = float(new_entry['resolution']) - resbin if float(new_entry['resolution']) >= resbin else 0.
        end = float(new_entry['resolution']) + resbin
        df_resbin = cdf[(cdf['resolution'] >= start) & (cdf['resolution'] <= end)]

        return df_resbin
    else:
        return None


def plot_bar_mat(a, b, qmin, qmax, qscore, work_dir, plot_name, score_type):
    """
    This function here using matplotlib to produce the Q-score bar image
    """

    a = a*1.5 if a else None
    b = b*1.5 if b else None
    a = a/200 if a else None
    b = b/200 if b else None
    # Create a color scale from 0 to 1
    color_scale = np.linspace(0, 1, 199)

    # Create a figure and axis
    fig, ax = plt.subplots(figsize=(8, 2), dpi=300)
    plt.rcParams['font.family'] = 'Times New Roman'

    # Reverse the color map
    cmap = plt.get_cmap("bwr")
    reversed_cmap = cmap.reversed()
    # Plot the color scale with a thinner aspect ratio
    ax.imshow([color_scale], cmap=reversed_cmap, aspect=0.05, extent=[0, 1.5, 0, 1])

    # Calculate the height and half-width of the diamonds
    diamond_height = 0.65
    diamond_half_width = 0.01

    # Add diamond-shaped marker for 'a'
    if a != b:
        ax.fill(
            [a - diamond_half_width, a, a + diamond_half_width, a],
            [0.5, 0.5 + diamond_height, 0.5, 0.5 - diamond_height],
            color='Black', edgecolor='black'
        )

        # Add diamond-shaped marker for 'b'
        ax.fill(
            [b - diamond_half_width, b, b + diamond_half_width, b],
            [0.5, 0.5 + diamond_height, 0.5, 0.5 - diamond_height],
            facecolor='none', edgecolor='black'
        )
    else:
        # ax.fill(
        #     [b - diamond_half_width, b, b + diamond_half_width, b],
        #     [0.5, 0.5 + diamond_height, 0.5, 0.5 - diamond_height],
        #     facecolor='yellow', edgecolor='black'
        # )
        top = np.array([[b-diamond_half_width, 0.5], [b, 0.5 + diamond_height], [b + diamond_half_width, 0.5], [b, 0.5]])
        bottom = np.array([[b-diamond_half_width, 0.5], [b, 0.5], [b + diamond_half_width, 0.5], [b, 0.5 - diamond_height]])
        top_patch = patches.Polygon(top, closed=True, facecolor='black', edgecolor='black')
        bottom_patch = patches.Polygon(bottom, closed=True, facecolor='none', edgecolor='black')
        ax.add_patch(top_patch)
        ax.add_patch(bottom_patch)

    # add four values as annotationso
    worse = r'$\it{Worse}$'
    better = r'$\it{Better}$'
    ax.annotate(worse, (0, -0.9), color='black', ha='left', fontsize=10, )
    ax.annotate(better, (1.5, -0.9), color='black', ha='right', fontsize=10)
    ax.annotate(f'{qscore:.3f}', (1.58, 0.2), color='black', ha='center', fontsize=12)

    if score_type == 'ccc':
        ax.annotate('CCC', (-0.11, 0.2), color='black', ha='center', fontsize=12)
    elif score_type == 'ai':
        ax.annotate('Atom inclusion', (-0.20, 0.2), color='black', ha='center', fontsize=12)
    elif score_type == 'smco':
        ax.annotate('SMOC', (-0.11, 0.2), color='black', ha='center', fontsize=12)
    elif score_type == 'qscore':
        ax.annotate('Q-score', (-0.11, 0.2), color='black', ha='center', fontsize=12)
    else:
        ax.annotate(score_type.upper(), (-0.11, 0.2), color='black', ha='center', fontsize=12)

    ax.annotate('Metric', (-0.11, 1.7), color='black', ha='center', fontsize=14)
    ax.annotate('Percentile Ranks', (0.75, 1.7), color='black', ha='center', fontsize=14)
    title = plot_name[:-8]
    ax.annotate(title, (0.75, 3.3), color='black', ha='center', fontsize=14, fontweight='bold')
    ax.annotate('Value', (1.58, 1.7), color='black', ha='center', fontsize=14)
    # if a >= b:
    #    ax.annotate(f'{a*100/1.5:.2f}%', (a, 1.4), color='black', ha='left', fontsize=10)
    #    #ax.annotate(f'{b:.2f}', (b, -0.8), color='black', ha='center', fontsize=10)
    #    #ax.annotate(f'{a*100:.2f}%', (a, 1.4), color='black', ha='center', fontsize=10)
    #    ax.annotate(f'{b*100/1.5:.2f}%', (b, 1.4), color='black', ha='right', fontsize=10)
    # else:
    #    ax.annotate(f'{a*100/1.5:.2f}%', (a, 1.4), color='black', ha='right', fontsize=10)
    #    #ax.annotate(f'{b:.2f}', (b, -0.8), color='black', ha='center', fontsize=10)
    #    #ax.annotate(f'{a*100:.2f}%', (a, 1.4), color='black', ha='center', fontsize=10)
    #    ax.annotate(f'{b*100/1.5:.2f}%', (b, 1.4), color='black', ha='left', fontsize=10)

    # Customize the plot
    ax.set_xlim(-0.4, 1.7)
    # ax.set_ylim(-4.3, 1.8)
    # to fit the EMD id
    ax.set_ylim(-4.3, 3.6)
    ax.set_yticks([])

    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    # Remove the left and bottom axis lines (optional)
    ax.spines['left'].set_visible(False)
    ax.spines['bottom'].set_visible(False)

    if a != b:
        # Add diamond-shaped marker for legend
        wa = 0.01
        ha = -2.0
        ax.fill(
            [wa - diamond_half_width, wa, wa + diamond_half_width, wa],
            [ha, ha + diamond_height, ha, ha - diamond_height],
            color='black', edgecolor='black'
        )
        ax.annotate(f'Percentile relative to all EM structures', (wa + 3 * diamond_half_width, ha - 0.25),
                    color='black', ha='left', fontsize=11)

        bwa = 0.01
        bha = -3.6
        ax.fill(
            [bwa - diamond_half_width, bwa, bwa + diamond_half_width, bwa],
            [bha, bha + diamond_height, bha, bha - diamond_height],
            facecolor='none', edgecolor='black'
        )
        ax.annotate('Percentile relative to EM structures of $\pm$1 $\mathrm{\AA}$ (resolution)',
                    (bwa + 3 * diamond_half_width, bha - 0.25), color='black', ha='left', fontsize=11)
        # ax.annotate(f'Percentile relative to EM structures of nearest 1000 (resolution)', (bwa + 3*diamond_half_width, bha-0.25), color='black', ha='left', fontsize=11)
    else:
        # Add diamond-shaped marker for legend
        wa = 0.01
        ha = -2.0
        ax.fill(
            [wa - diamond_half_width, wa, wa + diamond_half_width, wa],
            [ha, ha + diamond_height, ha, ha - diamond_height],
            color='black', edgecolor='black'
        )
        ax.annotate('Percentile relative to all EM structures (overlapped)', (wa + 3 * diamond_half_width, ha - 0.25),
                    color='black', ha='left', fontsize=11)
        # ax.annotate('Percentile relative to all EM structures (overlapped)', (wa + 3*diamond_half_width, ha-0.25), color='black', ha='left', fontsize=11)

        bwa = 0.01
        bha = -3.6
        ax.fill(
            [bwa - diamond_half_width, bwa, bwa + diamond_half_width, bwa],
            [bha, bha + diamond_height, bha, bha - diamond_height],
            facecolor='none', edgecolor='black'
        )
        ax.annotate('Percentile relative to EM structures of $\pm$1 $\mathrm{\AA}$ (resolution)',
                    (bwa + 3 * diamond_half_width, bha - 0.25), color='black', ha='left', fontsize=11)
        # ax.annotate(f'Percentile relative to EM structures of nearest 1000 (resolution)', (bwa + 3*diamond_half_width, bha-0.25), color='black', ha='left', fontsize=11)

    ax.tick_params(axis='both', which='both', length=0)
    plt.gca().set_xticklabels([])
    # Show the plot
    plot_name = '{}{}'.format(work_dir, plot_name)
    plt.savefig(plot_name)
    plt.close()


def bar(new_entry_dict, score_type, work_dir, score_dir, plot_name):
    input_file = '{}/qscores.csv'.format(score_dir)
    print(input_file)
    # new_entry_dict = {'id': '8117', 'resolution': 2.95, 'name': '5irx.cif', 'qscore': 0.521}
    qmin = None
    qmax = None
    df = load_score(input_file, new_entry_dict, score_type)
    if score_type and new_entry_dict[score_type]:
        (qmin, qmax), original_value = get_score(df, new_entry_dict[score_type], score_type)
        target_value = int(match_to_newscale((0, sum(original_value)), (0, 199), original_value[0]))
        to_whole = round(target_value/200., 3)
        to_whole_real = round((float(original_value[0]) / float(sum(original_value))), 3)
    else:
        to_whole = None
        to_whole_real = None

    #df1000 = get_nearest_onethousand(new_entry_dict, df, 500, score_type)
    if new_entry_dict['resolution']:
        df1000 = get_resolution_range(new_entry_dict, df, score_type)
        (sqmin, sqmax), ovalue = get_score(df1000, new_entry_dict[score_type], score_type)
        target_value_two = int(match_to_newscale((0, sum(ovalue)), (0, 199), ovalue[0]))
        to_two = round(target_value_two/200., 3)
        to_two_real = round((float(ovalue[0]) / float(sum(ovalue))), 3)
    else:
        to_two = None
        to_two_real = None

    if to_whole and to_two:
        plot_bar_mat(target_value, target_value_two, qmin, qmax, new_entry_dict[score_type], work_dir, plot_name, score_type)
    print(to_whole_real, to_two_real)
    return to_whole_real, to_two_real



#!/usr/bin/env python

"""
mainva.py

mainva class used to run the main function of validation
analysis

Copyright [2013] EMBL - European Bioinformatics Institute
Licensed under the Apache License, Version 2.0 (the
"License"); you may not use this file except in
compliance with the License. You may obtain a copy of
the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied. See the License for the
specific language governing permissions and limitations
under the License.

"""


__author__ = 'Zhe Wang'
__email__ = 'zhe@ebi.ac.uk'
__date__ = '2018-07-24'



import timeit
import os
import sys
from memory_profiler import memory_usage
from va.validationanalysis import ValidationAnalysis
from va.preparation import PreParation as prep
from va.metrics.strudel import run_strudel
import threading
try:
    from PATHS import MAP_SERVER_PATH
except ImportError:
    MAP_SERVER_PATH = None
from memory_profiler import profile
sys.stdout.flush()


def allruns(validationobj, runs):
    """
        Each single run included here

    :param validationobj: validation object from ValidationAnalysis module
    :param runs: list of string which represent each of the function
    :return: None
    """

    # Projections
    if 'projection' in runs:
        validationobj.new_projection()

    if 'projection' in runs:
        validationobj.orthogonal_projections()
        validationobj.rawmap_projections()
        validationobj.orthogonal_max()
        validationobj.rawmap_max()
        validationobj.orthogonal_min()
        validationobj.rawmap_min()
        validationobj.orthogonal_median()
        validationobj.rawmap_median()
        validationobj.orthogonal_std()
        validationobj.rawmap_std()

    # Predicated contour
    if 'predictcontour' in runs:
        validationobj.cl_prediction()

    # Central slice
    if 'central' in runs:
        validationobj.central_slice()
        validationobj.rawmap_central_slice()

    # Largest variance
    if 'largestvariance' in runs:
        validationobj.largest_variance()
        validationobj.rawmap_largest_variance()

    # Generate atom inclusion and residue inclusion json file
    if 'inclusion' in runs:
        validationobj.ai_bar()

    # Sureface views
    if 'surface' in runs:
        validationobj.surfaces()

    # Masks views
    if 'mask' in runs:
        validationobj.masks()

    # Desnity distribution
    if 'density' in runs:
        validationobj.mapdensity_distribution()
        validationobj.rawmapdensity_distribution()

    # Contour level verses volume
    if 'volume' in runs:
        validationobj.volumecontour()

    # rmcc/CCC
    if 'rmmcc' in runs:
        validationobj.real_mmcc()

    # RAPS
    if 'raps' in runs:
        validationobj.rapss()

    # FSC
    if 'fsc' in runs:
        validationobj.fscs()

    # mmFSC
    if 'mmfsc' in runs:
        validationobj.phenix_mmfsc()

    # Symmetry
    if 'symmetry' in runs:
        validationobj.symmetry()

    # Strudel
    if 'strudel' in runs:
        validationobj.strudel()

    # Q-score
    if 'qscore' in runs:
        validationobj.qscore_bar()

    # SMOC
    if 'smoc' in runs:
        validationobj.smoc_bar()

    # Phenix residue-wise CCC
    if 'resccc' in runs:
        validationobj.ccc_bar()
    #
    # if 'phrand' in runs:
    #     validationobj.phrand()

    # Resmap local resolution
    if 'locres' in runs:
        validationobj.locres_resmap()
        validationobj.locres_histo()
        validationobj.locres_residue()
        validationobj.model_map_ration()
        validationobj.locres_map()

    # EMringer
    if 'emringer' in runs:
        validationobj.emringer()

    # 3DFSC
    # if '3dfsc' in runs:
    #     validationobj.threedfsc()

    return None


def inallruns(validationobj, runs):
    """
        Each single run included here

    :param validationobj: validation object from ValidationAnalysis module
    :param runs: list of string which represent each of the function
    :return: None
    """

    def without_strudel():
        # Projections
        if 'projection' in runs:
            validationobj.orthogonal_projections()
            validationobj.rawmap_projections()
            validationobj.orthogonal_max()
            validationobj.orthogonal_std()

        # Central slice
        if 'central' in runs:
            validationobj.central_slice()
            validationobj.rawmap_central_slice()

        # Largest variance
        if 'largestvariance' in runs:
            validationobj.imgvariance()
            validationobj.rawmap_imgvariance()

        # Generate atom inclusion and residue inclusion json file
        if 'inclusion' in runs:
            validationobj.atom_inclusion()

        # Sureface views
        if 'surface' in runs:
            validationobj.surfaces()

        # Masks views
        if 'mask' in runs:
            validationobj.masks()

        # Desnity distribution
        if 'density' in runs:
            validationobj.mapdensity_distribution()
            validationobj.rawmapdensity_distribution()

        # Contour level verses volume
        if 'volume' in runs:
            validationobj.volumecontour()

        # rmcc/CCC
        if 'rmmcc' in runs:
            validationobj.real_mmcc()

        # RAPS
        if 'raps' in runs:
            validationobj.raps()
            validationobj.rawmap_raps()
            # validationobj.pararaps()

        # FSC
        if 'fsc' in runs:
            validationobj.fscs()

        # mmFSC
        if 'mmfsc' in runs:
            validationobj.mmfsc()

        # Symmetry
        if 'symmetry' in runs:
            validationobj.symmetry()

        # # Strudel
        # if 'strudel' in runs:
        #     validationobj.strudel()

        # Q-score
        if 'qscore' in runs:
            validationobj.qscore()

        # SMOC
        if 'smoc' in runs:
            validationobj.smoc()

        # EMringer
        if 'emringer' in runs:
            validationobj.emringer()

        # 3DFSC
        # if '3dfsc' in runs:
        #     validationobj.threedfsc()

        # Local resolution

        return None

    def strudel_only():
        # Strudel
        if 'strudel' in runs:
            validationobj.strudel()

        return None

    t = threading.Thread(target=strudel_only)
    t1 = threading.Thread(target=without_strudel)
    t.start()
    t1.start()
    t.join()
    t1.join()


# @profile
def main():

    start_first = timeit.default_timer()

    # Preparation
    prepobj = prep()
    position = prepobj.args.positions

    if not prepobj.args.positions:
        # Read map
        inputmap, mapsize, mapdimension = prepobj.read_map()

        # Read model
        inputmodel, pid, modelsize = prepobj.read_model()

        # Get model map names
        # modelsmaps, _ = prepobj.modelstomaps()
        modelsmaps = None
        onlybar = prepobj.onlybar

        # Read half map
        halfeven, halfodd, rawmap, halfmapsize = prepobj.read_halfmaps()

        # Contour level
        contourlevel = prepobj.contourlevel

        # Read run parameters
        runs = prepobj.runs()

        # EMDID
        emdid = prepobj.emdid
        dir = None if emdid is not None else prepobj.dir
        queue = prepobj.queue if emdid else None

        # Resolution and EM method
        resolution, emmethod = prepobj.resolution, prepobj.method

        # fscfile
        fscfile = prepobj.fscfile

        # strudel lib
        strudellib = prepobj.mofit_libpath

        # 3DFSC root directory
        threed_fsc_dir = prepobj.threedfscdir

        # Masks
        masks = prepobj.masks

        # Memory prediction
        memmsg = None
        # if len(runs) == 10 and 'mask' not in runs:
        memmsg = prepobj.memmsg(mapsize)

        # Platform either wwpdb or emdb(default)
        platform = prepobj.platform
        print('Executed under: {} platform.'.format(platform))

        # VA starts here on
        print('%sValidation Analysis%s' % (20*'=', 20*'='))
        validationobj = ValidationAnalysis(inputmap, inputmodel, pid, halfeven, halfodd, rawmap, contourlevel, emdid,
                                           dir, emmethod, resolution, fscfile, masks, modelsmaps, onlybar, strudellib,
                                           threed_fsc_dir, platform, queue)

        # Run all the validation piceces and give the peak memory consumption
        if len(runs) == 9 and 'mask' not in runs:
            mem = max(memory_usage((allruns, (validationobj, runs)), multiprocess=True, include_children=True))
            print('The memory peak is: {}.'.format(mem))
        else:
            mem = 0.
            allruns(validationobj, runs)

        # Change cif file to the original one
        prepobj.change_cifname()
        # Merge all jsons
        prepobj.finiliszejsons()

        stop = timeit.default_timer()
        alltime = stop - start_first
        # print('Memory usage peak: %s.' % mem)
        print('All time: %s' % alltime)


        # Save data for memory prediction
        #if runs['all'] is True:
        if len(runs) == 9 and 'mask' not in runs:
            if inputmodel is not None and modelsize is not None:
                modelout = sum(modelsize)/len(modelsize) if len(inputmodel) != 0 else 0
                vout = MAP_SERVER_PATH if emdid is not None else os.path.dirname(os.path.dirname(dir))
                with open(vout + '/input.csv', 'a+') as f:
                    if os.stat(vout + '/input.csv').st_size == 0:
                        f.write('%s,%s,%s,%s,%s,%s,%s\n' % ('mapname', 'maprealsize', 'halfmapsize', 'modelrealsize', 'mapdimension', 'alltime', 'mem'))
                    f.write('%s,%s,%s,%s,%s,%s,%s\n' % (prepobj.mapname, mapsize, halfmapsize, modelout, mapdimension, alltime, mem))
                f.close()
            else:
                vout = MAP_SERVER_PATH if emdid is not None else os.path.dirname(os.path.dirname(dir))
        else:
            vout = None

        return None
    elif position == 'strudel':
        run_strudel(prepobj.full_modelpath, prepobj.full_mappath, prepobj.mofit_libpath, prepobj.strdout, prepobj.queue)


if __name__ == '__main__':
    main()




import subprocess
import sys
import os
from distutils.spawn import find_executable
import pandas as pd

def create_folder(output_path):
    """
        create emringer output folder inside va directory

    :return: model related path of strudel folder
    """

    fullname = '{}'.format(output_path)

    if not os.path.isdir(fullname):
        os.mkdir(fullname, mode=0o777)
    else:
        print('{} is exist'.format(fullname))


def run_phenixcc(full_modelpath, full_mappath, resolution, output_path):
    """

        Phenix score

    :return:
    """

    errlist = []
    try:
        assert find_executable('phenix.map_model_cc') is not None
        phenixpath = find_executable('phenix.map_model_cc')
        create_folder(output_path)
        phenixcc_cmd = '{} {} {} resolution={}'.format(phenixpath, full_modelpath, full_mappath, resolution)
        print(phenixcc_cmd)
        try:
            process = subprocess.Popen(phenixcc_cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.STDOUT, cwd=output_path)
            output = process.communicate('n\n')[0]
            errqscore = 'error'
            if sys.version_info[0] >= 3:
                for item in output.decode('utf-8').split('\n'):
                    # item = cline.decode('utf-8').strip()
                    print(item)
                    if errqscore in item.lower():
                        errline = item.strip()
                        errlist.append(errline)
                        assert errqscore not in output.decode('utf-8'), errline

            else:
                for item in output.split('\n'):
                    print(item)
                    if errqscore in item.lower():
                        errline = item.strip()
                        errlist.append(errline)
                        assert errqscore not in output.decode('utf-8'), errline
            # post_phenixcc()
        except subprocess.CalledProcessError as suberr:
            err = 'Phenix residue-wise CCC calculation error: {}.'.format(suberr)
            errlist.append(err)
            sys.stderr.write(err + '\n')
    except AssertionError as exerr:
        err = 'Phenix executable is not there.'
        errlist.append(err)
        sys.stderr.write('Phenix executable is not there: {}\n'.format(exerr))

    return errlist


def read_cc(cc_per_residue_log, errlist):
    """
        Read the residue-wise CCC from the output of Phenix results
    :param: cc_per_residue_log input file cc_per_residue.log file(full path name) from Phenix CC calculation
    :return:
    """

    if not errlist and os.path.isfile(cc_per_residue_log):
        df = pd.read_csv(cc_per_residue_log, delim_whitespace=True, header=None)
        print(df)
    else:
        df = None

    return df


def _floatohex(numlist):
    """
        Todo: make this function into utlis
        Produce hex color between red and green
    :param numlist: A list of RGB values
    :return: A list of hex value between R and G with B = 0
    """

    numlist = [-1 if i < 0 else i for i in numlist]
    rgbs = [[122, int(num * 255), int(num * 255)] if num >= 0 else [255, 0, 255] for num in numlist]
    resultlist = ['#%02X%02X%02X' % (rgb[0], rgb[1], rgb[2]) for rgb in rgbs]

    return resultlist

def ccdf_todict(ccdf):
    """
        Given the Phenix CC result in dataframe and output the dict to be save in json
    :param ccdf: phenix cc_per_residue.log result into dataframe
    :return: dict which contain CC per residue results
    """

    finaldict = {}
    averagecc = ccdf[3].mean()
    averagecc_color = _floatohex([averagecc])[0]
    numberofresidues = ccdf.shape[0]
    colors = _floatohex(ccdf[3])
    ccscores = ccdf[3]
    chain_ccscores = ccdf.groupby(0).agg(value=(3, 'mean'))
    chain_ccscores['color'] = chain_ccscores.apply(lambda x: _floatohex([x['value']])[0], axis=1)
    chain_ccdict = {str(row.name): {'value': row['value'], 'color': row['color']} for _, row in chain_ccscores.iterrows()}

    def abc(a, b, c):
        return '{}:{} {}'.format(a, b, c)
    residue = ccdf.apply(lambda x: abc(x[0], x[2], x[1]), axis=1)
    finaldict = {'averagecc': round(averagecc, 3), 'averagecc_color': averagecc_color,
                         'numberofresidues': numberofresidues, 'color': colors,
                         'ccscore': ccscores.tolist(), 'residue': residue.tolist(), 'chainccscore': chain_ccdict}

    return finaldict




# -*- coding: utf-8 -*-
# setup.py
from setuptools import find_packages, setup
import sys
from va import __version__




with open('README.rst') as f:
    long_description = f.read()

if sys.version_info[0] == 2:
    cv = "opencv-python==4.2.0.32"
else:
    cv = "opencv-python"

setup(
    name="emdbva",
    version=__version__,
    packages=find_packages(),
    exclude_package_data={
        'va': ['test/*', 'va/onedepva.py', 'core/*', 'proc/*', 'validation/*', 'test_data/*'],
    },
    include_package_data=True,
    author="Zhe Wang",
    author_email="zhe@ebi.ac.uk",
    description="CryoEM validation toolkit",
    long_description=long_description,
    long_description_content_type='text/x-rst; charset=UTF-8',
    url="https://test.pypi.org/project/va/",
    license="Apache License",
    keywords="EMDB,Cryoem, Validation",
    # setup_requires=[
    #     "setuptools",
    #     "wheel",
    #     "twine",
    setup_requires=[
        'numpy'
    ],
    install_requires=[
        "numpy",
        "scipy>=0.14",
        "scikit-learn",
        "Pillow",
        "memory-profiler",
        "matplotlib",
        "pandas",
        "biopython",
        "mrcfile",
        "torch",
        cv,
    ],
    classifiers=[
        # maturity
        'Development Status :: 2 - Pre-Alpha',
        # environment
        'Environment :: Console',
        # audience
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        # license
        'License :: OSI Approved :: Apache Software License',
        # python version
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 2.7',
    ],
    entry_points={
        'console_scripts': [
            'va = va.mainva:main',
        ]
    },
)
